# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1 import utils


class TestUtils(object):
    def test_flatten(self):
        s = {'jim': 123, 'a': {'b': {'c': {'d': 'bob'}}}, 'rob': 34}
        r = utils.flatten(s)
        r.sort()
        assert r == ['a.b.c.d=bob', 'jim=123', 'rob=34']



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2016 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import jobstate


def test_context_updated(dci_context, job_id):
    assert dci_context.last_jobstate_id is None
    jobstate.create(dci_context, 'pre-run', 'comment', job_id)
    assert dci_context.last_jobstate_id is not None



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import base
from dciclient.v1 import utils


RESOURCE = 'files'
TABLE_HEADERS = ['id', 'name', 'content', 'mime', 'md5', 'jobstate_id',
                 'team_id', 'created_at']


def create(context, name, content, mime='text/plain',
           jobstate_id=None, md5=None, job_id=None):
    headers = {'DCI-NAME': name,
               'DCI-MIME': mime,
               'DCI-JOBSTATE-ID': jobstate_id,
               'DCI-MD5': md5,
               'DCI-JOB-ID': job_id}
    headers = utils.sanitize_kwargs(**headers)
    uri = '%s/%s' % (context.dci_cs_api, RESOURCE)
    return context.session.post(uri, headers=headers, data=content)


def get(context, id, where=None, embed=None):
    return base.get(context, RESOURCE, id=id, where=where, embed=embed)


def list(context, where=None, embed=None):
    return base.list(context, RESOURCE, where=None, embed=None)


def update(context, name=None, content=None, mime=None, md5=None):
    return base.update(context, RESOURCE, name=name, content=content,
                       mime=mime, md5=md5)


def delete(context, id):
    return base.delete(context, RESOURCE, id=id)


def content(context, id):
    uri = '%s/%s/%s/content' % (context.dci_cs_api, RESOURCE, id)
    r = context.session.get(uri)
    return r



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2016 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import click
import json
import prettytable
import six


def flatten(d, prefix=''):
    ret = []
    for k, v in d.items():
        p = k if not prefix else prefix + '.' + k
        if isinstance(v, dict):
            ret += flatten(v, prefix=p)
        else:
            ret.append("%s=%s" % (p, v))
    return ret


def print_json(result_json):
    formatted_result = json.dumps(result_json, indent=4)
    click.echo(formatted_result)


def print_prettytable(data, headers):
    DEFAULT_HEADERS = ['Property', 'Value']
    headers = headers or DEFAULT_HEADERS
    table = prettytable.PrettyTable(headers)

    if headers == DEFAULT_HEADERS:
        for item in sorted(six.iteritems(data)):
            table.add_row(item)
    else:
        if not isinstance(data, list):
            data = [data]

        for record in data:
            table.add_row([record[item] for item in headers])

    click.echo(table)


def sanitize_kwargs(**kwargs):
    boolean_fields = ['active']
    kwargs = dict(
        (k, v) for k, v in six.iteritems(kwargs) if k in boolean_fields or v
    )

    try:
        kwargs['data'] = json.loads(kwargs['data'])
    except KeyError:
        pass
    except TypeError:
        pass

    return kwargs


def format_output(result, format, item=None, headers=None,
                  success_code=(200, 201, 204)):

    result_json = result.json()
    if format == 'json' or result.status_code not in success_code:
        print_json(result_json)
    else:
        to_display = result_json[item] if item else result_json
        print_prettytable(to_display, headers)



# -*- encoding: utf-8 -*-
#
# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import dciclient.v1.api.file as dci_file
import dciclient.v1.helper as dci_helper


def test_run_command(dci_context, jobstate_id):
    dci_helper.run_command(
        dci_context,
        ['echo', 'bob'],
        jobstate_id=jobstate_id)
    # temporary debug print to trace an issue with the gate
    print(dci_file.list(dci_context).json())
    new_file = dci_file.list(dci_context).json()['files'][1]
    assert new_file['size'] == 4
    assert 'bob' in new_file['name']


def test_run_command_shell(dci_context, jobstate_id):
    dci_helper.run_command(
        dci_context,
        'echo foo bar',
        shell=True)
    files = dci_file.list(dci_context).json()['files']
    assert files[1]['name'] == 'echo foo bar'
    f = dci_file.content(dci_context, files[1]['id'])
    assert f.content.decode(encoding='UTF-8') == 'foo bar\n'



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import unicode_literals
import json


def test_prettytable_output(runner, team_id):
    result = runner.invoke(['user-create', '--name', 'foo',
                            '--password', 'pass', '--role', 'user',
                            '--team_id', team_id])
    user = json.loads(result.output)['user']

    result = runner.invoke(['--format', 'table', 'user-show', '--id',
                            user['id']])

    output = result.output.split('\n')
    # NOTE(spredzy) : The expected output for a table format looks like the
    #                 following :
    #
    # +------------
    # |  id | name
    # +------------
    # | id1 | name1
    # | id2 | name2
    # | id3 | name3
    # +------------
    #
    # The header variable below represents the header data, when more than one
    # space is located between the string and the '|' space number is shrink to
    # one ( this is what ' '.join(string.split()) does
    #
    # The data variable below represents the actual data, when more than one
    # space is located between the string and the '|' space number is shrink to
    # one ( this is what ' '.join(string.split()) does
    header = ' '.join(output[1].split())
    data = ' '.join(output[3].split())

    expected_data = (user['id'], user['etag'], user['name'], user['role'],
                     user['team_id'], user['created_at'], user['updated_at'])

    assert header == ('| id | etag | name | role | team_id '
                      '| created_at | updated_at |')

    assert data == '| %s | %s | %s | %s | %s | %s | %s |' % expected_data


def test_create(runner, team_id):
    result = runner.invoke(['user-create', '--name', 'foo',
                            '--password', 'pass', '--role', 'user',
                            '--team_id', team_id])
    user = json.loads(result.output)['user']
    assert user['name'] == 'foo'
    assert user['role'] == 'user'
    assert user['team_id'] == team_id


def test_list(runner, team_id):
    runner.invoke(['user-create', '--name', 'foo',
                   '--password', 'pass', '--role', 'user',
                   '--team_id', team_id])
    runner.invoke(['user-create', '--name', 'bar',
                   '--password', 'pass', '--role', 'user',
                   '--team_id', team_id])
    result = runner.invoke(['user-list'])
    users = json.loads(result.output)['users']
    # NOTE (spredzy): We put 5 because of the 3 creates plus
    # admin and 2 users provisionned during server test
    assert len(users) == 5


def test_update(runner, team_id):
    result = runner.invoke(['user-create', '--name', 'foo',
                            '--password', 'pass', '--role', 'user',
                            '--team_id', team_id])
    user = json.loads(result.output)['user']

    runner.invoke(['user-update', '--id', user['id'],
                   '--etag', user['etag'], '--name', 'bar',
                   '--role', 'admin'])
    result = runner.invoke(['user-show', '--id', user['id']])
    user = json.loads(result.output)['user']

    assert user['name'] == 'bar'
    assert user['role'] == 'admin'


def test_delete(runner, team_id):
    result = runner.invoke(['user-create', '--name', 'foo',
                            '--password', 'pass', '--role', 'user',
                            '--team_id', team_id])
    user = json.loads(result.output)['user']

    result = runner.invoke(['user-delete', '--id', user['id'],
                            '--etag', user['etag']])
    result = json.loads(result.output)
    assert result['message'] == 'User deleted.'


def test_show(runner, team_id):
    result = runner.invoke(['user-create', '--name', 'foo',
                            '--password', 'pass', '--role', 'user',
                            '--team_id', team_id])
    user = json.loads(result.output)['user']

    result = runner.invoke(['user-show', '--id', user['id']])
    user = json.loads(result.output)['user']

    assert user['name'] == 'foo'



# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import traceback

from dciclient.v1.api import file
from dciclient.v1.api import job
from dciclient.v1.api import jobdefinition
from dciclient.v1.api import jobstate
from dciclient.v1.logger import DciHandler

import json
import logging
import requests.packages.urllib3
import tripleohelper.undercloud


def push_stack_details(context, undercloud, stack_name='overcloud'):
    undercloud.yum_install(['git'])
    repo_url = 'https://github.com/goneri/tripleo-stack-dump'
    undercloud.run(
        'test -d tripleo-stack-dump || git clone ' + repo_url,
        user='stack')
    undercloud.add_environment_file(
        user='stack',
        filename='stackrc')
    undercloud.run(
        './tripleo-stack-dump/list_nodes_status',
        user='stack')
    undercloud.run(
        './tripleo-stack-dump/tripleo-stack-dump ' + stack_name,
        user='stack')
    with undercloud.open('/home/stack/tripleo-stack-dump.json') as fd:
        j = job.get(
            context,
            id=context.last_job_id).json()['job']
        job.update(
            context,
            id=context.last_job_id,
            etag=j['etag'],
            configuration=json.load(fd))


def run_tests(context, undercloud_ip, key_filename, user='root',
              stack_name='overcloud'):

    # redirect the log messages to the DCI Control Server
    # https://github.com/shazow/urllib3/issues/523
    requests.packages.urllib3.disable_warnings()
    dci_handler = DciHandler(context)
    logger = logging.getLogger('tripleohelper')
    logger.addHandler(dci_handler)

    undercloud = tripleohelper.undercloud.Undercloud(
        hostname=undercloud_ip,
        user=user,
        key_filename=key_filename)
    undercloud.create_stack_user()

    final_status = 'success'
    if undercloud.run(
            'test -f stackrc',
            user='stack',
            success_status=(0, 1,))[1] != 0:
        msg = 'undercloud deployment failure'
        jobstate.create(context, 'failure', msg, context.last_job_id)
        return
    push_stack_details(context, undercloud, stack_name=stack_name)
    rcfile = stack_name + 'rc'
    if undercloud.run(
            'test -f ' + rcfile,
            user='stack',
            success_status=(0, 1,))[1] != 0:
        msg = 'overcloud deployment failure'
        jobstate.create(context, 'failure', msg, context.last_job_id)
        return

    j = job.get(context, context.last_job_id).json()['job']
    tests = jobdefinition.get_tests(
        context, j['jobdefinition_id']).json()
    try:
        for t in tests['tests']:
            if 'url' not in t['data']:
                continue
            jobstate.create(
                context,
                'running',
                'Running test ' + t['name'],
                context.last_job_id)
            url = t['data']['url']
            undercloud.add_environment_file(
                user='stack',
                filename=rcfile)
            undercloud.run('curl -O ' + url, user='stack')
            undercloud.run('bash -x run.sh ' + stack_name, user='stack')
            with undercloud.open('result.xml', user='stack') as fd:
                file.create(
                    context,
                    t['name'],
                    fd.read(), mime='application/junit',
                    job_id=context.last_job_id)
    except Exception:
        msg = traceback.format_exc()
        final_status = 'failure'
        print(msg)
    else:
        msg = 'test(s) success'
    jobstate.create(context, final_status, msg, context.last_job_id)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import base
from dciclient.v1.api import jobdefinition

import json


RESOURCE = 'jobs'
TABLE_HEADERS = ['id', 'status', 'recheck', 'jobdefinition_id', 'remoteci_id',
                 'team_id', 'etag', 'created_at', 'updated_at']


def create(context, recheck, remoteci_id, team_id, jobdefinition_id=None,
           components=None, comment=None):
    job = base.create(context, RESOURCE, recheck=recheck,
                      remoteci_id=remoteci_id, team_id=team_id,
                      jobdefinition_id=jobdefinition_id,
                      components=components,
                      comment=comment)
    context.last_job_id = job.json()['job']['id']
    return job


def update(context, **kwargs):
    return base.update(context, RESOURCE, **kwargs)


def list(context):
    return base.list(context, RESOURCE)


def schedule(context, remoteci_id, topic_id):
    uri = '%s/%s/schedule' % (context.dci_cs_api, RESOURCE)
    data_json = json.dumps({'remoteci_id': remoteci_id, 'topic_id': topic_id})
    r = context.session.post(uri, data=data_json)
    if r.status_code == 201:
        context.last_job_id = r.json()['job']['id']
    return r


def recheck(context, id):
    uri = '%s/%s/%s/recheck' % (context.dci_cs_api, RESOURCE, id)
    r = context.session.post(uri)
    if r.status_code == 201:
        context.last_job_id = r.json()['job']['id']
    return r


def get(context, id, where=None, embed=None):
    return base.get(context, RESOURCE, id=id, where=where, embed=embed)


def list_results(context, id):
    uri = '%s/%s/%s/results' % (context.dci_cs_api, RESOURCE, id)
    return context.session.get(uri)


def get_full_data(context, id):
    # Get the job with embed on test and remoteci
    embed = 'jobdefinition,remoteci'
    job = base.get(context, RESOURCE, id=id, embed=embed).json()['job']
    # Get the components of the job
    job_components = get_components(context, id).json()['components']
    # Get the tests of the associated jobdefinition
    jobdefinition_tests = jobdefinition.get_tests(
        context, job['jobdefinition']['id']).json()['tests']

    # Aggregate the data of each resource
    full_data = {'remoteci': job['remoteci'],
                 'jobdefinition': job['jobdefinition'],
                 'tests': [],
                 'components': []}

    for component in job_components:
        full_data['components'].append(component)
    for test in jobdefinition_tests:
        full_data['tests'].append(test)

    return full_data


def get_components(context, id):
    uri = '%s/%s/%s/components' % (context.dci_cs_api, RESOURCE, id)
    return context.session.get(uri)


def delete(context, id, etag):
    return base.delete(context, RESOURCE, id=id, etag=etag)


def list_issues(context, id):
    uri = '%s/%s/%s/issues' % (context.dci_cs_api, RESOURCE, id)
    return context.session.get(uri)


def attach_issue(context, id, url):
    uri = '%s/%s/%s/issues' % (context.dci_cs_api, RESOURCE, id)
    data_json = json.dumps({'url': url})
    return context.session.post(uri, data=data_json)


def unattach_issue(context, id, issue_id):
    uri = '%s/%s/%s/issues/%s' % (context.dci_cs_api, RESOURCE, id, issue_id)
    return context.session.delete(uri)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import dci
import dci.app
import dci.common.utils as dci_utils
import dci.dci_config

import dciclient.shell as shell
import dciclient.v1 as dci_client
import dciclient.v1.api as api
import dciclient.v1.tests.shell_commands.utils as utils

import pytest
import sqlalchemy
import sqlalchemy_utils.functions


import click.testing
import functools
import passlib.apps as passlib_apps


@pytest.fixture(scope='session')
def engine(request):
    conf = dci.dci_config.generate_conf()
    db_uri = conf['SQLALCHEMY_DATABASE_URI']

    engine = sqlalchemy.create_engine(db_uri)

    def del_db():
        if sqlalchemy_utils.functions.database_exists(db_uri):
            sqlalchemy_utils.functions.drop_database(db_uri)

    del_db()
    request.addfinalizer(del_db)
    sqlalchemy_utils.functions.create_database(db_uri)

    dci.db.models.metadata.create_all(engine)
    return engine


@pytest.fixture
def db_clean(request, engine):
    def fin():
        for table in reversed(dci.db.models.metadata.sorted_tables):
            engine.execute(table.delete())
    request.addfinalizer(fin)


@pytest.fixture(scope='session', autouse=True)
def memoize_password_hash():
    pwd_context = passlib_apps.custom_app_context
    pwd_context.verify = dci_utils.memoized(pwd_context.verify)
    pwd_context.encrypt = dci_utils.memoized(pwd_context.encrypt)


@pytest.fixture
def db_provisioning(db_clean, engine):
    with engine.begin() as conn:
        utils.provision(conn)


@pytest.fixture
def server(db_provisioning, engine):
    app = dci.app.create_app(dci.dci_config.generate_conf())
    app.testing = True
    app.engine = engine
    return app


@pytest.fixture
def client(server, db_provisioning):
    client = dci_client.DCIClient(
        end_point='http://dci_server.com/api',
        login='admin', password='admin'
    )
    flask_adapter = utils.FlaskHTTPAdapter(server.test_client())
    client.s.mount('http://dci_server.com', flask_adapter)
    return client


@pytest.fixture
def dci_context(server, db_provisioning):
    test_context = api.context.DciContext('http://dci_server.com',
                                          'admin', 'admin')
    flask_adapter = utils.FlaskHTTPAdapter(server.test_client())
    test_context.session.mount('http://dci_server.com', flask_adapter)
    return test_context


@pytest.fixture
def dci_context_other_user_agent(server, db_provisioning):
    test_context = api.context.DciContext('http://dci_server.com',
                                          'admin', 'admin',
                                          user_agent='myagent-0.1')
    flask_adapter = utils.FlaskHTTPAdapter(server.test_client())
    test_context.session.mount('http://dci_server.com', flask_adapter)
    return test_context


@pytest.fixture
def dci_context_broken(server, db_provisioning):
    test_context = api.context.DciContext('http://no_where.com',
                                          'admin', 'admin')
    test_context.last_jobstate_id = 1
    test_context.last_job_id = 1
    return test_context


@pytest.fixture
def runner(dci_context):
    api.context.build_dci_context = lambda **kwargs: dci_context
    runner = click.testing.CliRunner(env={'DCI_LOGIN': '', 'DCI_PASSWORD': '',
                                          'DCI_CLI_OUTPUT_FORMAT': 'json'})
    runner.invoke = functools.partial(runner.invoke, shell.main)
    return runner


@pytest.fixture
def team_id(dci_context):
    return api.team.create(dci_context, name='tname').json()['team']['id']


@pytest.fixture
def topic_id(dci_context):
    kwargs = {'name': 'foo_topic'}
    return api.topic.create(dci_context, **kwargs).json()['topic']['id']


@pytest.fixture
def test_id(dci_context, topic_id):
    kwargs = {'name': 'test_name', 'topic_id': topic_id}
    return api.test.create(dci_context, **kwargs).json()['test']['id']


@pytest.fixture
def components(dci_context, topic_id):
    component1 = {'name': 'component1',
                  'type': 'git_commit',
                  'data': {},
                  'canonical_project_name': 'component 1',
                  'topic_id': topic_id}

    component2 = {'name': 'component2',
                  'type': 'git_commit',
                  'data': {},
                  'canonical_project_name': 'component 2',
                  'topic_id': topic_id}

    return [component1, component2]


@pytest.fixture
def remoteci_id(dci_context):
    team_id = api.team.create(dci_context, name='tname').json()['team']['id']
    kwargs = {'name': 'remoteci', 'team_id': team_id}
    rci = api.remoteci.create(dci_context, **kwargs).json()
    return rci['remoteci']['id']


@pytest.fixture
def component_id(dci_context, topic_id):
    kwargs = {'name': 'component1', 'type': 'git_review',
              'data': {'component': 'component'}, 'topic_id': topic_id}

    component = api.component.create(dci_context, **kwargs).json()
    return component['component']['id']


@pytest.fixture
def job_id(dci_context):
    JUNIT = """
    <testsuite errors="0" failures="0" name="pytest" skips="1"
               tests="3" time="46.050">
    <properties>
      <property name="x" value="y" />
      <property name="a" value="b" />
    </properties>
    <testcase classname="" file="test-requirements.txt"
              name="test-requirements.txt" time="0.0109479427338">
        <skipped message="all tests skipped by +SKIP option"
                 type="pytest.skip">Skipped for whatever reasons</skipped>
    </testcase>
    <testcase classname="tests.test_app" file="tests/test_app.py" line="26"
              name="test_cors_preflight" time="2.91562318802"/>
    <testcase classname="tests.test_app" file="tests/test_app.py" line="42"
              name="test_cors_headers" time="0.574683904648"/>
    </testsuite>"""

    topic = api.topic.create(dci_context, name='topic_name').json()['topic']
    team = api.team.create(dci_context, name='tname').json()['team']
    team_id = api.team.list(dci_context).json()['teams'][0]['id']

    api.topic.attach_team(dci_context, topic['id'], team_id)

    kwargs = {'name': 'tname', 'team_id': team['id'],
              'data': {'remoteci': 'remoteci'}}
    remoteci = api.remoteci.create(dci_context, **kwargs).json()
    remoteci_id = remoteci['remoteci']['id']

    kwargs = {'name': 'hihi', 'type': 'type_1', 'topic_id': topic['id'],
              'data': {'component': 'component1'}}
    api.component.create(dci_context, **kwargs).json()

    kwargs = {'name': 'haha', 'type': 'type_2', 'topic_id': topic['id'],
              'data': {'component': 'component2'}}
    api.component.create(dci_context, **kwargs).json()

    kwargs = {'name': 'tname', 'topic_id': topic['id'],
              'component_types': ['type_1', 'type_2']}
    api.jobdefinition.create(dci_context, **kwargs).json()

    job = api.job.schedule(dci_context, remoteci_id, topic['id']).json()
    api.file.create(dci_context, name='res_junit.xml', content=JUNIT,
                    mime='application/junit', job_id=job['job']['id'])

    return job['job']['id']


@pytest.fixture
def jobstate_id(dci_context, job_id):
    kwargs = {'job_id': job_id, 'status': 'running', 'comment': 'some comment'}
    jobstate = api.jobstate.create(dci_context, **kwargs).json()
    return jobstate['jobstate']['id']



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import click

from dciclient.v1.shell_commands import cli
from dciclient.v1 import utils

from dciclient.v1.api import user


@cli.command("user-list", help="List all users.")
@click.pass_obj
def list(context):
    """List all users.

    >>> dcictl user-list
    """
    result = user.list(context)
    utils.format_output(result, context.format,
                        user.RESOURCE, user.TABLE_HEADERS)


@cli.command("user-create", help="Create a user.")
@click.option("--name", required=True)
@click.option("--password", required=True)
@click.option("--role", help="'admin' or 'user'")
@click.option("--team_id", required=True)
@click.pass_obj
def create(context, name, password, role, team_id):
    """create(context, name, password, role, team_id)

    Create a user.

    >>> dcictl user-create [OPTIONS]

    :param string name: Name of the user [required]
    :param string password: Password for the user [required]
    :param string role: Role of user (admin or user)
    :param string team_id: ID of the team to attach this user to [required]
    """
    result = user.create(context, name=name, password=password,
                         role=role, team_id=team_id)
    utils.format_output(result, context.format, user.RESOURCE[:-1])


@cli.command("user-update", help="Update a user.")
@click.option("--id", required=True)
@click.option("--etag", required=True)
@click.option("--name")
@click.option("--password")
@click.option("--role", help="'admin' or 'user'")
@click.pass_obj
def update(context, id, etag, name, password, role):
    """update(context, id, etag, name, password, role)

    Update a user.

    >>> dcictl user-update [OPTIONS]

    :param string id: ID of the user to update [required]
    :param string etag: Entity tag of the user resource [required]
    :param string name: Name of the user
    :param string password: Password of the user
    :param string role: Role of the user (admin or user)
    """
    result = user.update(context, id=id, etag=etag, name=name,
                         password=password, role=role)

    if result.status_code == 204:
        utils.print_json({'id': id, 'message': 'User updated.'})
    else:
        utils.format_output(result, context.format)


@cli.command("user-delete", help="Delete a user.")
@click.option("--id", required=True)
@click.option("--etag", required=True)
@click.pass_obj
def delete(context, id, etag):
    """delete(context, id, etag)

    Delete a user.

    >>> dcictl user-delete [OPTIONS]

    :param string id: ID of the user to delete [required]
    :param string etag: Entity tag of the user resource [required]
    """
    result = user.delete(context, id=id, etag=etag)

    if result.status_code == 204:
        utils.print_json({'id': id, 'message': 'User deleted.'})
    else:
        utils.format_output(result, context.format)


@cli.command("user-show", help="Show a user.")
@click.option("--id", required=True)
@click.pass_obj
def show(context, id):
    """show(context, id)

    Show a user.

    >>> dcictl user-show [OPTIONS]

    :param string id: ID of the user to show [required]
    """
    result = user.get(context, id=id)
    utils.format_output(result, context.format,
                        user.RESOURCE[:-1], user.TABLE_HEADERS)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import unicode_literals
import json

from dciclient.v1.api import remoteci
from dciclient.v1.api import team


def test_prettytable_output(runner):
    result = runner.invoke(['team-create', '--name', 'foo'])
    team = json.loads(result.output)['team']
    result = runner.invoke(['remoteci-create', '--name', 'foo', '--team_id',
                            team['id']])
    remoteci = json.loads(result.output)['remoteci']

    result = runner.invoke(['--format', 'table', 'remoteci-show', '--id',
                            remoteci['id']])

    output = result.output.split('\n')
    # NOTE(spredzy) : The expected output for a table format looks like the
    #                 following :
    #
    # +------------
    # |  id | name
    # +------------
    # | id1 | name1
    # | id2 | name2
    # | id3 | name3
    # +------------
    #
    # The header variable below represents the header data, when more than one
    # space is located between the string and the '|' space number is shrink to
    # one ( this is what ' '.join(string.split()) does
    #
    # The data variable below represents the actual data, when more than one
    # space is located between the string and the '|' space number is shrink to
    # one ( this is what ' '.join(string.split()) does
    header = ' '.join(output[1].split())
    data = ' '.join(output[3].split())

    expected_data = (remoteci['id'], remoteci['name'], remoteci['data'],
                     remoteci['active'], remoteci['team_id'],
                     remoteci['etag'], remoteci['created_at'],
                     remoteci['updated_at'])

    assert header == ('| id | name | data | active | team_id | etag '
                      '| created_at | updated_at |')

    assert data == '| %s | %s | %s | %s | %s | %s | %s | %s |' % expected_data


def test_list(runner):
    result = runner.invoke(['team-create', '--name', 'foo'])
    team = json.loads(result.output)['team']

    runner.invoke(['remoteci-create', '--name', 'foo', '--team_id',
                   team['id']])
    runner.invoke(['remoteci-create', '--name', 'bar', '--team_id',
                   team['id']])
    result = runner.invoke(['remoteci-list'])
    remotecis = json.loads(result.output)['remotecis']

    assert len(remotecis) == 2
    assert remotecis[0]['name'] == 'foo'
    assert remotecis[1]['name'] == 'bar'


def test_create(runner):
    result = runner.invoke(['team-create', '--name', 'foo'])
    team = json.loads(result.output)['team']
    result = runner.invoke(['remoteci-create', '--name', 'foo', '--team_id',
                            team['id'], '--active'])
    remoteci = json.loads(result.output)['remoteci']
    assert remoteci['name'] == 'foo'
    assert remoteci['active'] is True


def test_update(runner):
    result = runner.invoke(['team-create', '--name', 'foo'])
    team = json.loads(result.output)['team']
    result = runner.invoke(['remoteci-create', '--name', 'foo', '--team_id',
                            team['id']])
    remoteci = json.loads(result.output)['remoteci']

    result = runner.invoke(['remoteci-update', '--id', remoteci['id'],
                            '--etag', remoteci['etag'], '--name', 'bar',
                            '--no-active'])
    result = json.loads(result.output)

    assert result['message'] == 'Remote CI updated.'
    assert result['id'] == remoteci['id']


def test_delete(runner):
    result = runner.invoke(['team-create', '--name', 'foo'])
    team = json.loads(result.output)['team']

    result = runner.invoke(['remoteci-create', '--name', 'foo', '--team_id',
                            team['id']])
    remoteci = json.loads(result.output)['remoteci']

    result = runner.invoke(['remoteci-delete', '--id', remoteci['id'],
                            '--etag', remoteci['etag']])
    result = json.loads(result.output)

    assert result['message'] == 'Remote CI deleted.'


def test_show(runner):
    result = runner.invoke(['team-create', '--name', 'foo'])
    team = json.loads(result.output)['team']
    result = runner.invoke(['remoteci-create', '--name', 'foo', '--team_id',
                            team['id']])
    remoteci = json.loads(result.output)['remoteci']

    result = runner.invoke(['remoteci-show', '--id', remoteci['id']])
    remoteci = json.loads(result.output)['remoteci']

    assert remoteci['name'] == 'foo'


def test_get_data(runner):
    result = runner.invoke(['team-create', '--name', 'foo'])
    team = json.loads(result.output)['team']
    result = runner.invoke(['remoteci-create', '--name', 'foo', '--team_id',
                            team['id']])
    remoteci = json.loads(result.output)['remoteci']

    result = runner.invoke(['remoteci-get-data', '--id', remoteci['id']])
    assert json.loads(result.output) == {}
    runner.invoke(['remoteci-update', '--id', remoteci['id'],
                   '--etag', remoteci['etag'], '--data', {'foo': 'bar'}])
    result = runner.invoke(['remoteci-get-data', '--id', remoteci['id']])
    assert json.loads(result.output) == {'foo': 'bar'}


def test_embed(dci_context):
    team_id = team.create(dci_context, name='teama').json()['team']['id']

    rci = remoteci.create(dci_context, name='boa', team_id=team_id).json()
    rci_id = rci['remoteci']['id']

    rci_with_embed = remoteci.get(dci_context,
                                  id=rci_id, embed=['team']).json()
    embed_team_id = rci_with_embed['remoteci']['team']['id']

    assert team_id == embed_team_id



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import io
import requests.adapters
import requests.models
import requests.utils

from dci import auth
from dci.db import models


class FlaskHTTPAdapter(requests.adapters.HTTPAdapter):

    def __init__(self, flask_client):
        super(FlaskHTTPAdapter, self).__init__(pool_connections=1)
        self.client = flask_client

    def init_poolmanager(self, *args, **kwargs):
        pass

    def build_response(self, req, resp):
        response = requests.models.Response()
        response.status_code = resp.status_code

        response.headers = resp.headers
        response.encoding = (
            requests.utils.get_encoding_from_headers(resp.headers) or
            'utf-8'
        )
        response.raw = io.BytesIO(resp.data)
        response.request = req
        response.connection = self
        return response

    def close(self):
        pass

    def send(self, request, stream=False, timeout=None, verify=True,
             cert=None, proxies=None):

        content_type = request.headers.pop('Content-Type', None)
        content_length = request.headers.pop('Content-Length', None)
        response = self.client.open(request.path_url,
                                    method=request.method.lower(),
                                    data=request.body,
                                    headers=dict(request.headers),
                                    content_type=content_type,
                                    content_length=content_length)
        return self.build_response(request, response)


def generate_componenttype(client):
    return client.post('/componenttypes', {'name': 'my_component_type'}).json()


def generate_job(client):
    componenttype = generate_componenttype(client)

    test = client.post('/tests', {'name': 'my_test'}).json()
    component = client.post('/components', {
        'name': 'my_component',
        'componenttype_id': componenttype['id'],
        'sha': 'some_sha',
        'canonical_project_name': 'my_project'}
    ).json()

    jobdefinition = client.post('/jobdefinitions', {
        'test_id': test['id'], 'priority': 0
    }).json()

    client.post('/jobdefinition_components', {
        'jobdefinition_id': jobdefinition['id'],
        'component_id': component['id']
    }).json()

    team = client.post('/teams', {
        'name': 'my_team'
    }).json()

    remoteci = client.post('/remotecis', {
        'team_id': team['id']
    }).json()

    job = client.post('/jobs', {
        'team_id': team['id'],
        'jobdefinition_id': jobdefinition['id'],
        'remoteci_id': remoteci['id'],
        'recheck': False
    }).json()

    return job


def provision(db_conn):
    def db_insert(model_item, **kwargs):
        query = model_item.insert().values(**kwargs)
        return db_conn.execute(query).inserted_primary_key[0]

    user_pw_hash = auth.hash_password('user')
    user_admin_pw_hash = auth.hash_password('user_admin')
    admin_pw_hash = auth.hash_password('admin')

    # Create teams
    team_admin_id = db_insert(models.TEAMS, name='admin')
    team_user_id = db_insert(models.TEAMS, name='user')

    # Create users
    db_insert(models.USERS, name='user', role='user', password=user_pw_hash,
              team_id=team_user_id)

    db_insert(models.USERS, name='user_admin', role='admin',
              password=user_admin_pw_hash, team_id=team_user_id)

    db_insert(models.USERS, name='admin', role='admin', password=admin_pw_hash,
              team_id=team_admin_id)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import job
from dciclient import version

import mock


def test_standard_headers(job_id, dci_context):
    with mock.patch('requests.sessions.Session.send'):
        job.get(dci_context, job_id)
        # the request that will be should be send
        prepared_request = dci_context.session.send.call_args[0]
        ua = 'python-dciclient_' + version.__version__
        assert prepared_request[0].headers['User-Agent'] == ua
        assert prepared_request[0].headers['Client-Version'] == (
            'python-dciclient_%s' % version.__version__
        )


def test_ua_headers(job_id, dci_context_other_user_agent):
    with mock.patch('requests.sessions.Session.send'):
        job.get(dci_context_other_user_agent, job_id)
        # the request that will be should be send
        prepared_request = (dci_context_other_user_agent.session.send
                            .call_args[0])
        assert prepared_request[0].headers['User-Agent'] == 'myagent-0.1'
        assert prepared_request[0].headers['Client-Version'] == (
            'python-dciclient_%s' % version.__version__
        )



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from dciclient.v1.api import jobstate
import dciclient.v1.logger

import pytest


def test_logger_no_context(dci_context):
    """a jobdefinition should exists first."""
    my_logger = logging.getLogger('__chainsaw__')
    handler = dciclient.v1.logger.DciHandler(dci_context)
    my_logger.addHandler(handler)
    record = logging.makeLogRecord({
        'lineno': 1,
        'levelno': logging.ERROR,
        'msg': 'bob'})
    with pytest.raises(dciclient.v1.logger.DciLogPushFailure):
        handler.emit(record)


def test_logger_server(dci_context, job_id):
    jobstate.create(dci_context, 'pre-run', 'comment', job_id)
    my_logger = logging.getLogger('__chainsaw__')
    handler = dciclient.v1.logger.DciHandler(dci_context)
    my_logger.addHandler(handler)
    record = logging.makeLogRecord({
        'lineno': 1,
        'levelno': logging.ERROR,
        'msg': 'bob'})
    handler.emit(record)


def test_logger_info_as_jobstate(dci_context, job_id):
    my_logger = logging.getLogger('__chainsaw__')
    handler = dciclient.v1.logger.DciHandler(
        dci_context, info_as_jobstate=True)
    my_logger.addHandler(handler)
    my_logger.info('bob')
    handler.emit(None)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import unicode_literals
import json


def test_prettytable_output(runner, topic_id):
    result = runner.invoke(['jobdefinition-create', '--name', 'foo',
                            '--topic_id', topic_id])
    jobdefinition = json.loads(result.output)['jobdefinition']

    result = runner.invoke(['--format', 'table', 'jobdefinition-show', '--id',
                            jobdefinition['id']])

    output = result.output.split('\n')
    # NOTE(spredzy) : The expected output for a table format looks like the
    #                 following :
    #
    # +------------
    # |  id | name
    # +------------
    # | id1 | name1
    # | id2 | name2
    # | id3 | name3
    # +------------
    #
    # The header variable below represents the header data, when more than one
    # space is located between the string and the '|' space number is shrink to
    # one ( this is what ' '.join(string.split()) does
    #
    # The data variable below represents the actual data, when more than one
    # space is located between the string and the '|' space number is shrink to
    # one ( this is what ' '.join(string.split()) does
    header = ' '.join(output[1].split())
    data = ' '.join(output[3].split())

    expected_data = (jobdefinition['id'], jobdefinition['name'],
                     jobdefinition['priority'], jobdefinition['active'],
                     jobdefinition['comment'], jobdefinition['etag'],
                     jobdefinition['created_at'], jobdefinition['updated_at'])

    assert header == ('| id | name | priority | active | comment '
                      '| etag | created_at | updated_at |')

    assert data == '| %s | %s | %s | %s | %s | %s | %s | %s |' % (
        expected_data
    )


def test_list(runner, topic_id):
    result = runner.invoke(['team-list'])
    teams = json.loads(result.output)['teams']
    team_id = teams[0]['id']

    topic_team = runner.invoke(['topic-attach-team', '--id', topic_id,
                                '--team_id', team_id])
    topic_team = json.loads(topic_team.output)

    runner.invoke(['jobdefinition-create', '--name', 'foo', '--topic_id',
                  topic_id])
    runner.invoke(['jobdefinition-create', '--name', 'bar', '--topic_id',
                  topic_id])

    result = runner.invoke(['jobdefinition-list', '--topic_id', topic_id])
    jobdefinitions = json.loads(result.output)['jobdefinitions']

    assert len(jobdefinitions) == 2
    assert jobdefinitions[0]['name'] == 'foo'
    assert jobdefinitions[1]['name'] == 'bar'


def test_create(runner, topic_id):
    result = runner.invoke(['jobdefinition-create', '--name', 'foo',
                            '--topic_id', topic_id])

    jobdefinition = json.loads(result.output)['jobdefinition']
    assert jobdefinition['name'] == 'foo'


def test_delete(runner, topic_id):
    result = runner.invoke(['jobdefinition-create', '--name', 'foo',
                            '--topic_id', topic_id])
    jobdefinition = json.loads(result.output)['jobdefinition']

    result = runner.invoke(['jobdefinition-delete', '--id',
                            jobdefinition['id'], '--etag',
                            jobdefinition['etag']])
    result = json.loads(result.output)

    assert result['message'] == 'Job Definition deleted.'


def test_show(runner, topic_id):
    result = runner.invoke(['jobdefinition-create', '--name', 'foo',
                            '--topic_id', topic_id])
    jobdefinition = json.loads(result.output)['jobdefinition']

    result = runner.invoke(['jobdefinition-show', '--id', jobdefinition['id']])
    jobdefinition = json.loads(result.output)['jobdefinition']

    assert jobdefinition['name'] == 'foo'


def test_annotate(runner, topic_id):
    result = runner.invoke(['jobdefinition-create', '--name', 'foo',
                            '--topic_id', topic_id])

    jd = json.loads(result.output)['jobdefinition']
    result = runner.invoke(['jobdefinition-annotate', '--id', jd['id'],
                            '--comment', 'This is my annotation', '--etag',
                            jd['etag']])

    result = json.loads(result.output)
    assert result['message'] == 'Job Definition updated.'

    result = runner.invoke(['jobdefinition-show', '--id', jd['id']])
    jobdefinition_comm = json.loads(result.output)['jobdefinition']['comment']
    assert jobdefinition_comm == 'This is my annotation'


def test_active(runner, topic_id):
    result = runner.invoke(['jobdefinition-create', '--name', 'foo',
                            '--topic_id', topic_id])

    jd = json.loads(result.output)['jobdefinition']
    result = runner.invoke(['jobdefinition-set-active', '--active', 'False',
                            '--id', jd['id'], '--etag', jd['etag']])

    result = json.loads(result.output)
    assert result['message'] == 'Job Definition updated.'
    result = runner.invoke(['jobdefinition-show', '--id', jd['id']])
    jobdefinition_active = json.loads(result.output)['jobdefinition']['active']
    assert jobdefinition_active is False



# -*- encoding: utf-8 -*-
#
# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import job


def test_job_updated(dci_context, job_id):
    new_configuration = {'foo': 'bar'}
    j = job.get(dci_context, job_id).json()['job']
    assert j['configuration'] == {}
    job.update(dci_context, id=job_id, etag=j['etag'],
               configuration=new_configuration)
    r = job.get(dci_context, job_id)
    j = r.json()['job']
    assert j['configuration'] == new_configuration



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import click

from dciclient.v1.shell_commands import cli
from dciclient.v1 import utils

from dciclient.v1.api import component
from dciclient.v1.api import topic

import json


@cli.command("component-list", help="List all components.")
@click.option("--topic_id", required=True)
@click.pass_obj
def list(context, topic_id):
    """list(context, topic_id)

    List all components.

    >>> dcictl component-list [OPTIONS]

    :param string topic_id: The topic ID for the list of components to return
    """
    components = topic.list_components(context, topic_id)
    utils.format_output(components, context.format,
                        component.RESOURCE, component.TABLE_HEADERS)


@cli.command("component-create", help="Create a component.")
@click.option("--name", required=True, help='Name of component')
@click.option("--type", required=True, help='Type of component')
@click.option("--topic_id", required=True, help='Topic ID')
@click.option("--canonical_project_name", help='Canonical project name')
@click.option("--data", default='{}', help='Data to pass (in JSON)')
@click.option("--sha", help='SHA hash')
@click.option("--title", help='Title of component')
@click.option("--message", help='Component message')
@click.option("--url", help='URL to look for the component')
@click.option("--git", help='Git reference for the component')
@click.option("--ref", help='External reference for the component')
@click.pass_obj
def create(context, name, type, canonical_project_name, data, sha,
           title, message, url, git, ref, topic_id):
    """create(context, name, type, canonical_project_name, data, sha, title,
message, url, git, ref, topic_id)

    Create a component.

    >>> dcictl component-create [OPTIONS]

    :param string name: Name of the component [required]
    :param string type: Type of the component [required]
    :param string topic_id: ID of the topic to associate with [required]
    :param string canonical_project_name: Project name
    :param json data: JSON to pass to the component
    :param string sha: SHA hash
    :param string title: Title of the component
    :param string message: Message for the component
    :param string url: URL resource to monitor
    :param string git: Git resource to monitor
    :param string ref: Reference resource to monitor
    """
    data = json.loads(data)
    result = component.create(
        context, name=name, type=type,
        canonical_project_name=canonical_project_name, data=data, sha=sha,
        title=title, message=message, url=url, git=git, ref=ref,
        topic_id=topic_id
    )
    utils.format_output(result, context.format, component.RESOURCE[:-1])


@cli.command("component-delete", help="Delete a component.")
@click.option("--id", required=True)
@click.pass_obj
def delete(context, id):
    """delete(context, id)

    Delete a component.

    >>> dcictl component-delete [OPTIONS]

    :param string id: ID of the component to delete [required]
    """
    result = component.delete(context, id=id)
    if result.status_code == 204:
        utils.print_json({'id': id, 'message': 'Component deleted.'})
    else:
        utils.format_output(result, context.format)


@cli.command("component-show", help="Show a component.")
@click.option("--id", required=True)
@click.pass_obj
def show(context, id):
    """show(context, id)

    Show a component

    >>> dcictl component-show [OPTIONS]

    :param string id: ID of the component to show [required]
    """
    result = component.get(context, id=id)
    utils.format_output(result, context.format, component.RESOURCE[:-1],
                        component.TABLE_HEADERS)



# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
import sys

import requests
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

from dciclient import version


class DciContext(object):
    API_VERSION = 'api/v1'

    def __init__(self, dci_cs_url, login, password, max_retries=0,
                 user_agent=None):
        self.session = self._build_http_session(login, password, user_agent)
        retries = Retry(total=max_retries,
                        backoff_factor=0.1)
        self.session.mount('http://', HTTPAdapter(max_retries=retries))
        self.session.mount('https://', HTTPAdapter(max_retries=retries))
        self.dci_cs_api = '%s/%s' % (dci_cs_url, DciContext.API_VERSION)
        self.last_jobstate_id = None
        self.last_job_id = None

    @staticmethod
    def _build_http_session(login, password, user_agent):
        session = requests.Session()
        session.headers.setdefault('Content-Type', 'application/json')
        if not user_agent:
            user_agent = 'python-dciclient_%s' % version.__version__
        session.headers['User-Agent'] = user_agent
        session.headers['Client-Version'] = (
            'python-dciclient_%s' % version.__version__
        )
        session.auth = (login, password)
        return session


def build_dci_context(dci_cs_url=None, dci_login=None, dci_password=None,
                      user_agent=None, max_retries=80):
    dci_cs_url = dci_cs_url or os.environ.get('DCI_CS_URL', '')
    dci_login = dci_login or os.environ.get('DCI_LOGIN', '')
    dci_password = dci_password or os.environ.get('DCI_PASSWORD', '')

    if not dci_cs_url or not dci_login or not dci_password:
        print("Environment variables required: DCI_CS_URL=%s, "
              "DCI_LOGIN=%s, DCI_PASSWORD=%s" %
              (dci_cs_url, dci_login, dci_password))
        sys.exit(1)
    return DciContext(dci_cs_url, dci_login, dci_password,
                      user_agent=user_agent, max_retries=max_retries)
